//
// Copyright (c) 2023 Ted Unangst <tedu@tedunangst.com>
//
// Permission to use, copy, modify, and distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package main

import (
	"fmt"
	"os"
	"strings"

	"humungus.tedunangst.com/r/webs/junk"
)

var importInProgress = false

func exportdata(what, where string) {
	var fd *os.File
	var err error
	if where == "-" {
		fd = os.Stdout
	} else {
		fd, err = os.OpenFile(where, os.O_CREATE|os.O_WRONLY|os.O_EXCL, 0666)
		if err != nil {
			elog.Fatal(err)
		}
		defer fd.Close()
	}
	post := findpost(what)
	if post == nil {
		elog.Fatal("export what?")
	}
	group := findgroup(post.GroupName)
	comments := getcomments(post.ID)
	reformatcomments(comments)
	persons := make(map[string]*Person)
	poster := findperson(post.PersonName)
	persons[poster.Name] = poster
	for _, c := range comments {
		if persons[c.PersonName] == nil {
			person := findperson(c.PersonName)
			persons[person.Name] = person
		}
	}
	j := junk.New()
	{
		gj := jsonGroup(group)
		name := gj["preferredUsername"].(string)
		if idx := strings.IndexByte(name, '@'); idx > 0 {
			name = name[0:idx]
		}
		gj["preferredUsername"] = name
		j["Group"] = gj
	}
	j["Post"] = jsonPost(post)
	var jp []junk.Junk
	for _, p := range persons {
		pj := jsonPerson(p)
		if pj["id"] == "." {
			pj["id"] = "https://azorius.test/p/deleted"
		}
		if pj["preferredUsername"] == "[deleted]" {
			pj["preferredUsername"] = "deleted"
		}
		name := pj["preferredUsername"].(string)
		if idx := strings.IndexByte(name, '@'); idx > 0 {
			name = name[0:idx]
		}
		pj["preferredUsername"] = name
		jp = append(jp, pj)
	}
	j["Persons"] = jp
	var jc []junk.Junk
	for _, c := range comments {
		c.PostXID = post.Xid
		cj := jsonComment(c)
		if cj["id"] == "." {
			cj["id"] = fmt.Sprintf("https://azorius.test/c/%d", c.ID)
		}
		if cj["inReplyTo"] == "." {
			cj["inReplyTo"] = fmt.Sprintf("https://azorius.test/c/%d", c.ParentID)
		}
		if cj["attributedTo"] == "." {
			cj["attributedTo"] = "https://azorius.test/p/deleted"
		}
		jc = append(jc, cj)
	}
	j["Comments"] = jc
	j.Write(fd)
}

func importdata(where string) {
	fd, err := os.Open(where)
	if err != nil {
		elog.Fatal(err)
	}
	defer fd.Close()
	j, err := junk.Read(fd)
	if err != nil {
		elog.Fatal(err)
	}
	importInProgress = true
	gj, _ := j.GetMap("Group")
	gid, _ := gj.GetString("id")
	domain := originate(gid)
	group := apHandleGroup(domain, gj, false)
	origin := Origin{Domain: domain, Group: group}
	pa, _ := j.GetArray("Persons")
	for _, pj := range pa {
		pj := pj.(junk.Junk)
		pid, _ := pj.GetString("id")
		domain = originate(pid)
		apHandlePerson(domain, pj)
	}
	pj, _ := j.GetMap("Post")
	apHandlePage(&origin, pj)
	ca, _ := j.GetArray("Comments")
	for _, cj := range ca {
		apHandleNote(&origin, cj.(junk.Junk))
	}
}
