//
// Copyright (c) 2023 Ted Unangst <tedu@tedunangst.com>
//
// Permission to use, copy, modify, and distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package main

import (
	notrand "math/rand"
	"time"
)

var bgnow = time.Now()

func refresher() {
	go refreshbgnow()
	for {
		sleep := time.Hour
		sleep += time.Duration(notrand.Int63n(int64(sleep)))
		time.Sleep(sleep)
		groups := getgroups()
		now := time.Now()
		ilog.Printf("starting a big refresh")
		for _, group := range groups {
			if group.IsLocal() {
				continue
			}
			age := int(now.Sub(group.Refreshed) / time.Hour)
			r := 16 + notrand.Intn(72)
			dlog.Printf("group %s is %d (/%d) hours old", group.Name, age, r)
			if age < r {
				continue
			}
			refreshGroup(group)
		}
	}
}

func refreshbgnow() {
	for {
		time.Sleep(10 * time.Minute)
		bgnow = time.Now()
	}
}

func refreshGroup(group *Group) {
	defer func() {
		if err := recover(); err != nil {
			elog.Printf("error refreshing group: %s", err)
		}
	}()
	dlog.Printf("going to refresh %s", group.Name)
	apid := group.APid
	j, err := fetchActivity(apid, fastTimeout)
	if err == nil {
		apHandleGroup(originate(apid), j, true)
		if group.Outbox != "" {
			apPullOutbox(group, group.Outbox)
		}
	} else {
		ilog.Printf("error refreshing group %s: %s", group.Name, err)
		group = dbFindGroup(group.Name)
		group.Refreshed = time.Now()
		dbSaveGroup(group)
	}
}
